/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.  All rights reserved.
 * 
 * Unpublished rights (if any) reserved under the copyright laws of the
 * United States of America and other countries.
 * 
 * This code is proprietary to MIPS Technologies, Inc. ("MIPS
 * Technologies").  Any copying, reproducing, modifying or use of this
 * code (in whole or in part) that is not expressly permitted in writing
 * by MIPS Technologies or an authorized third party is strictly
 * prohibited.  At a minimum, this code is protected under unfair
 * competition and copyright laws.  Violations thereof may result in
 * criminal penalties and fines.
 * 
 * MIPS Technologies reserves the right to change this code to improve
 * function, design or otherwise.  MIPS Technologies does not assume any
 * liability arising out of the application or use of this code, or of
 * any error or omission in such code.  Any warranties, whether express,
 * statutory, implied or otherwise, including but not limited to the
 * implied warranties of merchantability or fitness for a particular
 * purpose, are excluded.  Except as expressly provided in any written
 * license agreement from MIPS Technologies or an authorized third party,
 * the furnishing of this code does not give recipient any license to any
 * intellectual property rights, including any patent rights, that cover
 * this code.
 * 
 * This code shall not be exported or transferred for the purpose of
 * reexporting in violation of any U.S. or non-U.S. regulation, treaty,
 * Executive Order, law, statute, amendment or supplement thereto.
 * 
 * This code constitutes one or more of the following: commercial
 * computer software, commercial computer software documentation or other
 * commercial items.  If the user of this code, or any related
 * documentation of any kind, including related technical data or
 * manuals, is an agency, department, or other entity of the United
 * States government ("Government"), the use, duplication, reproduction,
 * release, modification, disclosure, or transfer of this code, or any
 * related documentation of any kind, is restricted in accordance with
 * Federal Acquisition Regulation 12.212 for civilian agencies and
 * Defense Federal Acquisition Regulation Supplement 227.7202 for
 * military agencies.  The use of this code by the Government is further
 * restricted in accordance with the terms of the license agreement(s)
 * and/or applicable contract terms and conditions covering this code
 * from MIPS Technologies or an authorized third party.
 */

/* 
 * sbd.h: MIPS Technologies Atlas board definition header file
 */


#ifndef MHZ
/* fastest possible pipeline clock */
#define MHZ		133
#endif

#define RAMCYCLE	60			/* 60ns dram cycle */
#define ROMCYCLE	150			/* ~150ns rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

/*
 * Address Map
 */

#define DRAM_BASE	0x00000000	/* 128MB */
#define DRAM_SIZE	 0x08000000
#define PCI_MEM_SPACE	0x08000000	/* 256MB: s/w configurable */
#define PCI_MEM_SIZE	 0x10000000
#define PCI_IO_SPACE	0x18000000	/*  62MB: s/w configurable */
#define PCI_IO_SIZE	 0x03e00000
#define GT_BASE		0x1be00000
#define SYSFLASH_BASE	0x1c000000
#define SYSFLASH_SIZE	 0x2000000
#define MONFLASH_BASE	0x1e000000
#define MONFLASH_SIZE	 0x1000000
#define FPGA_BASE	0x1f000000
#define BOOTPROM_BASE	0x1fc00000
#define BOOTPROM_SIZE	  0x400000
#define REVISION_ADDR	0x1fc00010

/* interrupt controller */
#define INTRAW_ADDR	(FPGA_BASE + 0x00000000)	/* raw inputs */
#define INTSETEN_ADDR	(FPGA_BASE + 0x00000008)	/* set enables */
#define INTRSTEN_ADDR	(FPGA_BASE + 0x00000010)	/* clear enabled */
#define INTENABLE_ADDR	(FPGA_BASE + 0x00000018)	/* enables */
#define INTSTATUS_ADDR	(FPGA_BASE + 0x00000020)	/* (input&enable) */
#define NMISTATUS_ADDR	(FPGA_BASE + 0x00000024)	/* nmp status */
#define HWINTACK_ADDR	(FPGA_BASE + 0x00000100)	/* interrupt ack */
#define NMIACK_ADDR	(FPGA_BASE + 0x00000104)	/* nmi ack */

/* device request/enable bits in interrupt registers */
#define INTREQ_SERR	0x00080000
#define INTREQ_INTD	0x00040000
#define INTREQ_INTC	0x00020000
#define INTREQ_INTB	0x00010000
#define INTREQ_INTA	0x00008000
#define INTREQ_ATXFAIL	0x00004000
#define INTREQ_DEG	0x00002000
#define INTREQ_ENUM	0x00001000
#define INTREQ_PCID	0x00000800
#define INTREQ_PCIC	0x00000400
#define INTREQ_PCIB	0x00000200
#define INTREQ_PCIA	0x00000100
#define INTREQ_CORELO	0x00000040
#define INTREQ_COREHI	0x00000020
#define INTREQ_RTC	0x00000010
#define INTREQ_TIM0	0x00000002
#define INTREQ_UART0	0x00000001


/* switch readback register */
#define SWITCH_ADDR	(FPGA_BASE + 0x00000200)

#define SWITCH_S14	0x00000080	/* switch S1-4 */
#define SWITCH_S13	0x00000040	/* switch S1-3 */
#define SWITCH_FLSHPROT 0x00000040	/* switch S1-3 - flash wr prot */
#define SWITCH_S12	0x00000020	/* switch S1-2 */
#define SWITCH_S11	0x00000010	/* switch S1-1 */
#define SWITCH_MONBOOT	0x00000010	/* switch S1-1 - boot from mon */
#define SWITCH_S2	0x0000000f	/* switch S2-7:0 */

/* board status register (r/o) */
#define STATUS_ADDR	(FPGA_BASE + 0x00000208)

#define STATUS_MFWR	0x00000010	/* monitor flash write enabled */
#define STATUS_S54	0x00000008	/* switch S5-4: reset environ */
#define STATUS_S53	0x00000004	/* switch S5-3 */
#define STATUS_BIGEND	0x00000002	/* S5-2: big endian */
#define STATUS_PCISYS	0x00000001	/* compact pci system controller */

/* jumpbers readback register */
#define JMPRS_ADDR	(FPGA_BASE + 0x00000210)

#define JMPRS_EELOCK	0x00000002	/* EEPROM write protected */
#define JMPRS_PCI33M	0x00000001	/* PCI 1=33MHx, 0=16.5MHz */

/* FPGA 40NHz onchip timer */
#define TM0CNT_ADDR	(FPGA_BASE + 0x00000300)
#define TM0CMP_ADDR	(FPGA_BASE + 0x00000308)
#define TMINTACK_ADDR	(FPGA_BASE + 0x00000320)
#define TM0HZ		40000000

/* green led */
#define LEDGREEN_ADDR	(FPGA_BASE + 0x00000400)

/* 8-bar led array */
#define LEDBAR_ADDR	(FPGA_BASE + 0x00000408)

/* 8-character ASCII led array */
#define ASCIIWORD_ADDR	(FPGA_BASE + 0x00000410)
#define ASCII_BASE	(FPGA_BASE + 0x00000418)
#define  ASCII_LED(n)	((n)*8)

/* software reset control */
#define SOFTRES_ADDR	(FPGA_BASE + 0x00000500)
#define SOFTRES_DATA	0x42

/* break signal reset timer */
#define BRKRES_ADDR	(FPGA_BASE + 0x00000508)

/* power-supply dtandby control */
#define PSUSTBY_ADDR	(FPGA_BASE + 0x00000600)
#define PSUSTBY_DATA	0x4d

/* flash write-enable */
#define SFWCTRL_ADDR	(FPGA_BASE + 0x00000700)
#define SFWCTRL_DATA	0xc7

/* Dallas DS1687-5 real-time clock */ 
#define RTC_BASE	(FPGA_BASE + 0x00000800)
#define  RTC_ADDR	(RTC_BASE + 0)
#define  RTC_DATA	(RTC_BASE + 8)

/* TI 16c550 uart */
#define UART0_BASE	(FPGA_BASE + 0x00000900)

/* local gpio */
#define GPOUT_ADDR	(FPGA_BASE + 0x00000a00)
#define GPIN_ADDR	(FPGA_BASE + 0x00000a08)


/* 
 * Console UART definitions
 */
#define NS16550_HZ	3686400

#define NSREG(x)	((x)*8)
#ifdef __ASSEMBLER__
#define NS16550_INB(dst,offs,base) \
	lw	dst,offs(base)
#define NS16550_OUTB(src,offs,base) \
	sw	src,offs(base)
#else
#define NS16550_INB(base, offs) \
	*((volatile unsigned int *)((char *)(base) + (offs)))
#define NS16550_OUTB(base, offs, data) \
	do { \
	  *((volatile unsigned int *)((char *)(base) + (offs))) = (data); \
	} while (0)
#endif

/* board attributes */
#define DEFSPEED	38400
/*#define _SBD_PANEL*/

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

#ifndef __ASSEMBLER__
extern void _sbd_nsdelay (unsigned long);
#define nsdelay(ns)	_sbd_nsdelay (ns)
#define usdelay(us)	_sbd_nsdelay ((us) * 1000)
#endif
