/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.  All rights reserved.
 * 
 * Unpublished rights (if any) reserved under the copyright laws of the
 * United States of America and other countries.
 * 
 * This code is proprietary to MIPS Technologies, Inc. ("MIPS
 * Technologies").  Any copying, reproducing, modifying or use of this
 * code (in whole or in part) that is not expressly permitted in writing
 * by MIPS Technologies or an authorized third party is strictly
 * prohibited.  At a minimum, this code is protected under unfair
 * competition and copyright laws.  Violations thereof may result in
 * criminal penalties and fines.
 * 
 * MIPS Technologies reserves the right to change this code to improve
 * function, design or otherwise.  MIPS Technologies does not assume any
 * liability arising out of the application or use of this code, or of
 * any error or omission in such code.  Any warranties, whether express,
 * statutory, implied or otherwise, including but not limited to the
 * implied warranties of merchantability or fitness for a particular
 * purpose, are excluded.  Except as expressly provided in any written
 * license agreement from MIPS Technologies or an authorized third party,
 * the furnishing of this code does not give recipient any license to any
 * intellectual property rights, including any patent rights, that cover
 * this code.
 * 
 * This code shall not be exported or transferred for the purpose of
 * reexporting in violation of any U.S. or non-U.S. regulation, treaty,
 * Executive Order, law, statute, amendment or supplement thereto.
 * 
 * This code constitutes one or more of the following: commercial
 * computer software, commercial computer software documentation or other
 * commercial items.  If the user of this code, or any related
 * documentation of any kind, including related technical data or
 * manuals, is an agency, department, or other entity of the United
 * States government ("Government"), the use, duplication, reproduction,
 * release, modification, disclosure, or transfer of this code, or any
 * related documentation of any kind, is restricted in accordance with
 * Federal Acquisition Regulation 12.212 for civilian agencies and
 * Defense Federal Acquisition Regulation Supplement 227.7202 for
 * military agencies.  The use of this code by the Government is further
 * restricted in accordance with the terms of the license agreement(s)
 * and/or applicable contract terms and conditions covering this code
 * from MIPS Technologies or an authorized third party.
 */

/* 
 * sbd.h: MIPS Technologies SEAD-2 board definition header file
 */


#ifndef MHZ
/* fastest possible pipeline clock */
#define MHZ		50
#endif

#define RAMCYCLE	60			/* 60ns dram cycle */
#define ROMCYCLE	150			/* ~150ns rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

/*
 * Address Map
 */

#define DRAM_BASE	0x00000000	/* 32MB DRAM */
#define DRAM_SIZE	 0x2000000
#define FLASH_BASE	0x1c000000	/* 32MB Flash */
#define FLASH_SIZE	  0x800000
#define FPGA_BASE	0x1f000000
#define BOOTPROM_BASE	0x1fc00000	/* Boot region */
#define BOOTPROM_SIZE	  0x400000
#define REVISION_ADDR	0x1fc00010

/* NMI controller */
#define NMISTATUS_ADDR	(FPGA_BASE + 0x00000040)	/* nmp status */
#define NMIACK_ADDR	(FPGA_BASE + 0x00000048)	/* nmi ack */
#define NMI_WERR	0x00000002
#define NMI_FLAG	0x00000001

/* software reset */
/* software reset control */
#define SOFTRES_ADDR	(FPGA_BASE + 0x00000050)
#define SOFTRES_DATA	0x42

/* green led */
/*#define LEDGREEN_ADDR	(FPGA_BASE + 0x00000400)*/

/* 8-bar led array */
#define LEDBAR_ADDR	(FPGA_BASE + 0x00000210)

/* 8-character ASCII led array */
/*#define ASCIIWORD_ADDR	(FPGA_BASE + 0x00000410)*/
#define ASCII_BASE	(FPGA_BASE + 0x000005c0)
#define  ASCII_LED(n)	((n)*8)

/* 
 * UART definitions
 */
#define UART0_BASE	(FPGA_BASE + 0x00000800)
#define UART1_BASE	(FPGA_BASE + 0x00000900)
#define NS16550_HZ	3686400
#define NSREG(x)	((x)*8)		/* regs mapped every 8 bytes */

/* use word accesses for endian independence */
#ifdef __ASSEMBLER__
#define NS16550_INB(dst,offs,base) \
	lw	dst,offs(base)
#define NS16550_OUTB(src,offs,base) \
	sw	src,offs(base)
#else
#define NS16550_INB(base, offs) \
	*((volatile unsigned int *)((char*)(base) + (offs)))
#define NS16550_OUTB(base, offs, data) \
	do { \
	  *((volatile unsigned int *)((char*)(base) + (offs))) = (data); \
	} while (0)
#endif

/* board attributes */
#define DEFSPEED	38400
/*#define _SBD_PANEL*/

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

#ifndef __ASSEMBLER__
extern void _sbd_nsdelay (unsigned long);
#define nsdelay(ns)	_sbd_nsdelay (ns)
#define usdelay(us)	_sbd_nsdelay ((us) * 1000)
#endif
